breeds    [acid                ;;parent acid molecuels
     hydroxide            ;;base molecules    
     hydronium            ;;hydronium molecules
     water                ;;water molecules
     con-base]            ;;conjugate base molecules
        

globals  [
    pH pOH  ;;variables for calculating pH
    volume
    clock   ;;clock variable   
    ]

turtles-own [id]                       ;;variable to control reactions

;;observer procedure to setup the system
to setup
    ca
    if see-starting-water? [ create-custom-water 100 [set shape "WATER-SHAPE"] ]        ;;creates constant volume of water
    create-custom-acid starting-acid [set shape "ACID-SHAPE"]        ;;creates variable amount of acid
    create-custom-con-base starting-conjbase [set shape "CON-BASE-SHAPE"]    ;;creates variable amount of conjugate base
    
    ;;randomizes position and headings
    ask turtles [rt random 360 setxy random screen-size-x random screen-size-y]

    ;;initializes amount of dissociated acid
    ask acid [if (random 100) > 98 [dissociate]]
    setup-plot                           
    calculate-pH    
    set volume count turtles
    set clock  0
    collide                                ;;tells turtles to find others on their patch
end

;;main turtle procedure
to go
  collide
  ask turtles [lt random 20 rt random 20 fd 1]
  calculate-pH
  plot-pH     
  plot-molecules
  set clock  (clock + 1)
end

;;turtle procedure to tell turtle how to find other turtles on the same patch

to collide
    ask hydroxide [set id random-one-of hydronium-here react 
                    set id random-one-of acid-here react]
    ask hydronium [set id random-one-of hydroxide-here react 
                   set id random-one-of con-base-here react]
end

;;turtle procedure that tells turtles how to react

to react            

if id != nobody
[ without-interruption [ 
  ;;all hydroxide molecules will instantly react with hydronium molecules and form water
  ifelse ((breed-of id) = hydronium)
    [set breed-of id water set shape-of id "WATER-SHAPE"
     set breed water set shape "WATER-SHAPE"]

  [ifelse ((breed-of id) = hydroxide)
    [set breed-of id water set shape-of id "WATER-SHAPE"
     set breed water set shape "WATER-SHAPE"]

  ;;hydroxide molecules have a high probability of reacting with acid molecules
  [ifelse ((breed-of id) = acid)
    [if ((random 100) > 80)
      [set breed-of id con-base set shape-of id "CON-BASE-SHAPE"
       set breed water set shape "WATER-SHAPE"]]
    ;;hydronium ions will react with conjugate base to form acid
  
  [if ((breed-of id) = con-base)
    [if ((random 100) > 80)
     [set breed-of id acid set shape-of id "ACID-SHAPE"
     set breed water set shape "WATER-SHAPE"]]
] ] ] ] ] 
end

;;turtle procedure for an acid molecule to dissociate into hydronium and con-base
to dissociate
  hatch 1 [set breed hydronium set shape "HYDRONIUM-SHAPE"]
  set breed con-base set shape "CON-BASE-SHAPE"
end


;;observer procedure that determines the pH

to calculate-pH
  locals [ concH concOH ]
  set volume count turtles 
  set concH (count hydronium / volume)
  set concOH (count hydroxide / volume)
  ifelse (concH = concOH)
    [ set pH 7 ]
    [ ifelse (concH > concOH)
      [ set pH (0 - log (concH / 1000) 10) ]
      [ set pOH (0 - log (concOH / 1000) 10)
        set pH 14 - pOH ] ]
end


;;turtle procedure used to add more base to the system
to add-base
    create-custom-hydroxide added-base
      [fd 1
       set id who
       rt random 360
       set shape "HYDROXIDE-SHAPE"]
end

;;turtle procedure used to add more acid to the system
to add-acid
    create-custom-hydronium added-acid
      [fd 1
       set id who
       rt random 360
       set shape "HYDRONIUM-SHAPE"]
end

;;plotting procedures;;

to setup-plot
    set-current-plot "pH"
    set-plot-x-range 0 100
    set-plot-y-range 0 14
    auto-plot-on
    set-plot-pen-color red
   
end

to plot-pH
    set-current-plot "pH"
    plot pH
end

to plot-molecules
       set-current-plot "number of molecules"
       set-current-plot-pen "conj-base"
       plot count con-base
       set-current-plot-pen "acid"
       plot count acid
       set-current-plot-pen "base"
       plot count hydroxide
       set-current-plot-pen "hydronium"
       plot count hydronium  
       set-current-plot-pen "water"
       plot count water
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo Buffer model.
; http://ccl.northwestern.edu/netlogo/models/Buffer.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
304
10
689
416
12
12
15.0
1
10
0
0

CC-WINDOW
690
85
951
220
Command Center

SLIDER
42
128
285
161
starting-conjbase
starting-conjbase
1
200
100
1
1
NIL

SLIDER
128
170
289
203
added-acid
added-acid
1
100
30
1
1
NIL

BUTTON
45
49
146
82
setup
setup
NIL
1
T
OBSERVER

BUTTON
157
49
287
82
go
go
T
1
T
OBSERVER

BUTTON
32
169
104
202
add acid
add-acid
NIL
1
T
OBSERVER

BUTTON
32
214
104
247
add base
add-base
NIL
1
T
OBSERVER

PLOT
32
299
289
495
pH
Time
pH
0.0
100.0
0.0
14.0
true
false

SLIDER
128
213
289
246
added-base
added-base
1
100
30
1
1
NIL

PLOT
304
427
898
646
number of molecules
time
#
0.0
100.0
0.0
10.0
true
true
PENS
"hydronium" 1.0 0 -11352576 true
"conj-base" 1.0 0 -65281 true
"base" 1.0 0 -65536 true
"acid" 1.0 0 -2500352 true
"water" 1.0 0 -16776961 true

SLIDER
42
92
286
125
starting-acid
starting-acid
1
200
100
1
1
NIL

SWITCH
67
258
241
291
see-starting-water?
see-starting-water?
1
1
-1000

MONITOR
62
568
144
617
pH
pH
2
1

MONITOR
158
569
237
618
clock
clock
0
1

MONITOR
62
515
144
564
# hydronium
count hydronium
0
1

MONITOR
158
514
237
563
# hydroxide
count hydroxide
0
1

@#$#@#$#@
WHAT IS IT?
-----------

This model demonstrates the behavior of a buffered solution.  A buffer is a solution that resists change in acidity (or pH) when either acid or base are added into it, within limits.  It is best viewed as the third model in the ACID-BASE package. 

Chemists and biologists use the properties of acids and bases to create buffer solutions.  It is often desirable to keep the acidity, or hydronium ion concentration of a solution, as nearly constant as possible.  Examples of such situation would be: measuring the velocity of a reaction, selectively dissolving certain salts, or studying the growth of bacteria or plants.   In our body, the blood has a buffering capacity, keeping the pH at 7.35.  If the pH goes above 7.7 (alkalosis) or below 7.0 (acidosis), the results are fatal.

To accomplish this feat, buffers depend on the presence of a weak acid and its conjugate base. With both of these species in the solution, additional acids and bases are absorbed according to the following chemical equations. H-A denotes the weak acid and A- denotes its conjugate base.

|          -                 -	 	 -     +	   -    
| H-A  + OH  ----->   H O + A		A + H O  -------> H O + H-A
| 		     2			     3		   2

The pH of a buffer is determined in the same manner that the pH of a weak acid is determined - by counting the number of hydromium and hydroxide ions and calculating their ratio withe total number of ions and molecules in the solution.

When there are more hydroniums,  pH = - log ( hydronium concentration )

When there are more hydroxides,  pH = 14 - pOH = 14 - log ( hydroxide concentration )

Buffers are effective only within their unique buffering range. This range is determined by the concentrations of the weak acid and its conjugate base.  Outside of its buffering range, the solution behaves as a strong acid or base.




HOW TO USE IT
--------------

Decide how many acid molecules should be present at the start of the simulation with the STARTING-ACID slider. Set the number of conjugate base molecules with the STARTING-CONJBASE slider.    

Press SETUP.   The turtles will distribute randomly across the screen. 

YELLOW turtles are acid molecules (HA)
PINK turtles are conjugate base molecules (A-)
GREEN turtles represent hydronium ions (H30+) 
RED turtles are hydroxide molecules (OH-)
BLUE turtles represent water molecules (H20)

In this model we are assuming that 2% of the original acid molecules are dissociated into 2 conjugate base molecule and 2 hydronium molecules.  This is true only before the molecules start interacting between themselves.

Press GO. The molecules will move randomly across the screen.  

When two turtles occupy the same patch, the following rules apply:
1.  When a weak acid and a water molecule collide, the acid dissociates to its conjugate base and the water molecules transforms to a hydronium ion.
2.  When hydroxide and hydronium ions collide, they always form two water molecules.
3.  When weak acid and hydroxide collide, they have a high probability of transforming into a conjugate base and a water molecule.
4.  When hydronium and conjugate base collide, they have a high probability of transforming into a weak acid molecule and a water molecule.

To observe the effect of adding base or acid to the solution, set the number of acid molecules you want to add with the ADDED-ACID slider and press ADD-ACID (H30+)
Do the same for adding base with the ADDED-BASE slider and ADD-BASE button.

A number of plots and monitors can be observed:
The pH of the solution is plotted over time on the PH CURVE, and PH MONITOR.  pH is calculated using the ratio of the number of hydronmium and hydroxide molecules to the total number of turtles.  This is different from the chemistry calculation that relates this number to solution volume.
You can see the number of hydroniums and hydroxides in the solution in their monitors  (# HYDRONIUMS, #  HYDROXIDES). 
You may follow the number of molecules of each species over time in the NUMBER OF MOLECULES plot.

You may choose to see the initial water molecules or not with the SEE-STARTING-WATER? Switch.


THINGS TO NOTICE
----------------

Observe the pH curve.  How are the numbers of hydronmium and hydroxide molecules in the monitors below related to the pH?

Examine the shape of the pH curve. Notice how the pH changes with respect to the amount of base or acid added. Does the buffer resist change in pH?

Observe the number of molecules in the plot.   When acid or base is added to the solution, they quickly disappear as they react. Can you determine which turtles react with each other?  What is the relationship between the two plots: pH and number of molecules?

What happens when large amounts of base or acid are added to the system? Is it the same as adding small amounts? Does the pH curve reflect this?


THINGS TO TRY

-------------

Add a large amount of acid or base to the solution and observe the effect. Why does the pH change dramatically outside of the buffering range? Would it be useful to add large amounts of base all at once to a solution in the laboratory if you were trying to adjust the pH?

Can you relate the idea of a buffering range to the molecules and their behavior in the model?

Try running the model with different amounts of acid or conjugate base. The buffering range of the solution should shift. How is the shift related to the changes you made? Does changing the proportion between the number of weak acid and conjugate base molecules have an effect on the buffering capacity?

How does changing the amount of buffer molecules influence the buffering capacity?

Try pressing SETUP a number of times with the same initial settings, and observe the number of hydroniums.  Is the number always the same? Press GO and watch the change in pH.  Why is the same equilibrium reached in each case?


EXTENDING THE MODEL
------------------

SIncrease the amount of hydronium originally generated upon SETUP by increasing the chance of dissociation in the procedures. What effect does this have on the initial pH?

Add a button and procedure to add more conjugate base to the reaction. How could this help a chemist who is trying to keep the pH of a solution at a constant value?

Add a procedure that allows you to plot a titration curve for a buffer. Is it similar to any other titration curves you have seen? What additional information can you can from the titration curve?

Additional interactions could take place in a solution in addition to those stated in the rules.  These were omitted because their probability of occurring is low.  Try including one of these rules to the procedure.  For example, two water molecules could transform into  a  hydronium and hydroxide ions.   Does it make a change?


CREDITS AND REFERENCES
----------------------
Thanks to Mike Stieff for his work on this model.

To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo Buffer model. http://ccl.northwestern.edu/netlogo/models/Buffer. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

acid-shape
true
0
Circle -256 true false 73 74 156
Circle -1 true false 48 199 55
Circle -1 true false 197 198 58

arrow
true
0
Polygon -7566196 true true 150 300 300 150 195 150 195 7 105 7 105 150 105 150 0 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

circle
true
0
Circle -7566196 true true 35 35 230

con-base-shape
true
0
Circle -65281 true false 78 79 146
Circle -1 true false 42 194 66

hydronium-shape
true
0
Circle -11352576 true false 91 92 118
Circle -1 true false 66 187 50
Circle -1 true false 186 186 52
Circle -1 true false 127 43 49

hydroxide-shape
true
0
Circle -65536 true false 80 80 142
Circle -1 true false 63 199 55

water-shape
true
0
Circle -16776961 true false 78 79 146
Circle -1 true false 40 41 69
Circle -1 true false 194 42 71

@#$#@#$#@
NetLogo 1.3beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
